from __future__ import annotations
from cnnecharts.mapping import Mapping

from typing import TYPE_CHECKING, List

# if TYPE_CHECKING:
#     from easyChart.layer import Layer


# class ChartOptions(object):
#     def __init__(self, mapping: Mapping) -> None:
#         self.mapping = mapping
#         self.layer: List[Layer] = []

#     def add(self, layer: Layer):
#         self.layer.append(layer)
#         return self
